emc_parcelPointLinks = {
    trigger: null,
    mapContainer: null,
    map: null,
    markers: [],
    getCarrierCallback: null,
    setPointCallback: null,
    refreshCarriersCallback: null,

    init: function (trigger, getCarrierCallback, getInitialAddress, setPointCallback, refreshCarriersCallback) {
        const self = this;
        self.trigger = trigger;
        self.getCarrierCallback = getCarrierCallback;
        self.getInitialAddress = getInitialAddress;
        self.setPointCallback = setPointCallback;
        self.refreshCarriersCallback = refreshCarriersCallback;
        self.on("body", "click", self.trigger, function() {
            var target = this;
            self.refreshCarriersCallback(target).then(function() {
                self.mapContainer = document.querySelector("#emcMap");
                if (!self.mapContainer) {
                    self.initMap();
                }
                self.openMap();
                self.getInitialAddress(target).then(function(address) {
                    self.fillAddressFields(address);
                    self.getPoints(address);
                });
            });
        });
        
        self.on("body", "click", "#validateZipCodeButton", function() {
            var zipCodeInput = document.querySelector('#zipCodeInput');
            var cityInput = document.querySelector('#cityInput');
            var countryInput = document.querySelector('#countryInput');
            var zipCode = zipCodeInput.length === 0 ? "" : zipCodeInput.value;
            var city = cityInput.length === 0 ? "" : cityInput.value;
            var country = countryInput.length === 0 ? "" : countryInput.value;
            self.clearMarkers();
            self.getPoints({
                "zipcode": zipCode, 
                "city": city, 
                "country": country
            });
        });

        self.on("body", "click", ".parcelButton", function() {
            self.setPointCallback(this.getAttribute("data-code"), this.getAttribute("data-name"), this.getAttribute("data-carrier"))
                .then(function(name) {
                    self.closeMap();
                })
                .catch(function(err) {
                    self.showError(err);
                });
        });
    },
    
    initMap: function() {
        var self = this;
        var mapClose = document.createElement("div");
        mapClose.setAttribute("class", "emcClose");
        mapClose.setAttribute("title", emc_lang['close map']);
        mapClose.addEventListener("click", function() {
            self.closeMap()
        });

        var mapCanvas = document.createElement("div");
        mapCanvas.setAttribute("id", "mapCanvas");

        var zipCodeInput = document.createElement("input");
        zipCodeInput.setAttribute("id", "zipCodeInput");
        zipCodeInput.setAttribute("name", "zipCodeInput");
        zipCodeInput.setAttribute("type", "text");
        zipCodeInput.setAttribute("placeholder", emc_lang.zipcode);
        
        var cityInput = document.createElement("input");
        cityInput.setAttribute("id", "cityInput");
        cityInput.setAttribute("name", "cityInput");
        cityInput.setAttribute("type", "text");
        cityInput.setAttribute("placeholder", emc_lang.city);
        
        var countryInput = document.createElement("input");
        countryInput.setAttribute("id", "countryInput");
        countryInput.setAttribute("name", "countryInput");
        countryInput.setAttribute("type", "hidden");
        
        var validateZipCodeButtonDesktopText = document.createElement("span");
        validateZipCodeButtonDesktopText.setAttribute("id", "validateZipCodeButtonDesktopText");
        validateZipCodeButtonDesktopText.innerHTML = emc_lang.searchDesktopText;
        
        var validateZipCodeButtonMobileText = document.createElement("span");
        validateZipCodeButtonMobileText.setAttribute("id", "validateZipCodeButtonMobileText");
        validateZipCodeButtonMobileText.innerHTML = emc_lang.searchMobileText;
        
        var validateZipCodeButton = document.createElement("button");
        validateZipCodeButton.setAttribute("id", "validateZipCodeButton");
        validateZipCodeButton.appendChild(validateZipCodeButtonDesktopText);
        validateZipCodeButton.appendChild(validateZipCodeButtonMobileText);
        
        var zipCodeContainer = document.createElement("div");
        zipCodeContainer.setAttribute("id", "zipCodeContainer");
        zipCodeContainer.appendChild(zipCodeInput);
        zipCodeContainer.appendChild(cityInput);
        zipCodeContainer.appendChild(countryInput);
        zipCodeContainer.appendChild(validateZipCodeButton);
        
        var mapInner = document.createElement("div");
        mapInner.setAttribute("id", "mapContainer");
        mapInner.appendChild(mapCanvas);

        var mapPPContainer = document.createElement("div");
        mapPPContainer.setAttribute("id", "prContainer");

        var mapOuter = document.createElement("div");
        mapOuter.setAttribute("id", "emcMapInner");
        mapOuter.appendChild(mapClose);
        mapOuter.appendChild(zipCodeContainer);
        mapOuter.appendChild(mapInner);
        mapOuter.appendChild(mapPPContainer);

        self.mapContainer = document.createElement("div");
        self.mapContainer.setAttribute("id", "emcMap");
        self.mapContainer.appendChild(mapOuter);
        document.body.appendChild(self.mapContainer);

        self.map = new mapboxgl.Map({
            container: 'mapCanvas',
            style: emc_mapUrl,
            zoom: 14,
            accessToken: 'whatever'
        });
        self.map.addControl(new mapboxgl.NavigationControl());

        var logoImg = document.createElement("img");
        logoImg.setAttribute("src", emc_mapLogoImageUrl);
        var logoLink = document.createElement("a");
        logoLink.setAttribute("href", emc_mapLogoHrefUrl);
        logoLink.setAttribute("target", "_blank");
        logoLink.appendChild(logoImg);
        var logoContainer = document.createElement("div");
        logoContainer.setAttribute("id", "emcBoxtalLogo");
        logoContainer.appendChild(logoLink);

        var mapTopLeftCorner = document.querySelector(".mapboxgl-ctrl-top-left");
        if (mapTopLeftCorner) {
            mapTopLeftCorner.appendChild(logoContainer);
        }
    },

    openMap: function() {
        this.mapContainer.classList.add("emcModalShow");
        var offset = window.pageYOffset + (window.innerHeight - this.mapContainer.offsetHeight)/2;
        if (offset < window.pageYOffset) {
            offset = window.pageYOffset;
        }
        this.mapContainer.style.top = offset + 'px';
        
        this.map.resize();
    },

    closeMap: function() {
        this.mapContainer.classList.remove("emcModalShow");
        this.clearMarkers();
    },

    getPoints: function(address) {
        var self = this;

        self.getParcelPoints(address).then(function(parcelPointResponse) {
            self.addParcelPointMarkers(parcelPointResponse);
            self.fillParcelPointPanel(parcelPointResponse);
            self.setMapBounds();
        }).catch(function(err) {
            self.showError(err);
        });
    },

    getParcelPoints: function(address) {
        var self = this;
        return new Promise(function(resolve, reject) {
            var httpRequest = new XMLHttpRequest();
            httpRequest.onreadystatechange = function() {
                if (httpRequest.readyState === 4) {
                    var response = typeof httpRequest.response === 'object' && httpRequest.response !== null ? httpRequest.response : JSON.parse(httpRequest.response);
                    if (response.success === false) {
                        reject(emc_lang["Unable to load parcel points"]);
                    } else {
                        resolve(response);
                    }
                }
            };
            httpRequest.open("POST", emc_ajaxUrl);
            httpRequest.setRequestHeader(
                "Content-Type",
                "application/x-www-form-urlencoded"
            );
            httpRequest.responseType = "json";
            httpRequest.send("action=emc_get_points&carrier_code=" + encodeURIComponent(self.getCarrierCallback()) 
            + "&zipcode=" + encodeURIComponent(address.zipcode) 
            + "&city=" + encodeURIComponent(address.city)
            + "&address=" + encodeURIComponent(address.address)
            + "&country=" + encodeURIComponent(address.country));
        });
    },

    addParcelPointMarkers: function(parcelPoints) {
        for (var i = 0; i < parcelPoints.length; i++) {
            parcelPoints[i].index = i;
            this.addParcelPointMarker(parcelPoints[i]);
        }
    },

    addParcelPointMarker: function(point) {
        var self = this;
        
        var choose = "";
        if (point.code) {
            var ope = point.code.substring(0, 4);
            if (emc_lang.relayName[ope]) {
                choose = emc_lang.relayName[ope];
            } else {
                choose = emc_lang.relayName.default;
            }
        }
        
        var info ="<div class='emcMakerPopup'><b>"+point.name+'</b><br/>'+
		    '<a href="#" class="parcelButton emcPointer" data-code="'+point.code+'" data-name="'+point.name+'" data-carrier="'+this.getCarrierCallback()+'">'+choose+'</a><br/>' +
			point.address + ', ' + point.zipcode + ' ' + point.city + '<br/>' + "<b>" + emc_lang['Opening hours'] +
            "</b><br/>"+'<div class="emcSchedule">';

        for (var j in point.schedule) {
			day = point.schedule[j];

            /* Test to prevent js errors */
            if ((typeof(day.open_am)=="undefined") || (typeof(day.close_am)=="undefined") ||
                (typeof(day.open_pm)=="undefined") || (typeof(day.close_pm)=="undefined")) {
                // Point not found, skipping...
                continue;
            }

			am = day.open_am !== "" && day.close_am !== "";
			pm = day.open_pm !== "" && day.close_pm !== "";
			if (am || pm) {
				info += '<span class="emcDay">'+emc_lang['day_'+day.weekday]+'</span>';
				if (am) {
					info += self.formatHours(day.open_am) +'-'+self.formatHours(day.close_am);
				}
				if (pm) {
					if (am) {
						info += ', '+self.formatHours(day.open_pm) +'-'+self.formatHours(day.close_pm);
					}
					else {
						info += self.formatHours(day.open_pm) +'-'+self.formatHours(day.close_pm);
					}
				}
                info += '<br/>';
			}
		}
		info += '</div>';

        var el = this.getMarkerHtmlElement(point.index + 1);

        var popup = new mapboxgl.Popup({ offset: 25 })
            .setHTML(info);

        var marker = new mapboxgl.Marker({
            element: el,
            anchor: 'bottom'
        })
            .setLngLat(new mapboxgl.LngLat(parseFloat(point.longitude), parseFloat(point.latitude)))
            .setPopup(popup)
            .addTo(self.map);

        self.markers.push(marker);

        self.addRightColMarkerEvent(marker, point.code);
    },

    addRightColMarkerEvent: function(marker, code) {
        this.on("body", "click", ".emc-show-info-" + code, function(){
            marker.togglePopup();
        });
    },

    formatHours: function(time) {
        var explode = time.split(':');
        if (3 === explode.length) {
            time = explode[0]+':'+explode[1];
        }
        return time;
    },

    setMapBounds: function() {

        var bounds = new mapboxgl.LngLatBounds();

        for (var i = 0; i < this.markers.length; i++) {
            var marker = this.markers[i];
            bounds = bounds.extend(marker.getLngLat());
        }

        this.map.fitBounds(
            bounds,
            {
                padding: 30,
                linear: true
            }
        );
    },

    fillParcelPointPanel: function(parcelPoints) {        
        var html = '';
        html += '<table><tbody>';
        for (var i = 0; i < parcelPoints.length; i++) {
            var point = parcelPoints[i];
            
            var choose = "";
            if (point.code) {
                var ope = point.code.substring(0, 4);
                if (emc_lang.relayName[ope]) {
                    choose = emc_lang.relayName[ope];
                } else {
                    choose = emc_lang.relayName.default;
                }
            }

            html += '<tr>';
            html += '<td><div class="emcPointHeader">' + this.getMarkerHtmlElement(parseInt(i)+1).outerHTML;
            html += '<div class="emcPointTitle"><a class="emc-show-info-' + point.code + ' emcPointer">' + point.name + '</a></div></div>';
            html += point.address + '<br/>';
            html += point.zipcode + ' ' + point.city + '<br/>';
            html += '<a class="parcelButton emcPointer" data-code="'+point.code+'" data-name="'+point.name+'" data-carrier="'+this.getCarrierCallback()+'"><b>'+choose+'</b></a>';
            html += '</td>';
            html += '</tr>';
        }
        html += '</tbody></table>';
        document.querySelector('#prContainer').innerHTML = html;
    },

    getMarkerHtmlElement: function(index) {
        var el = document.createElement('div');
        el.className = 'emcMarker';
        el.innerHTML = index;
        return el;
    },
    
    fillAddressFields: function(address) {
        var zipCodeInput = document.querySelector('#zipCodeInput');
        var cityInput = document.querySelector('#cityInput');
        var countryInput = document.querySelector('#countryInput');
        if (zipCodeInput !== null) {
            zipCodeInput.value = address.zipcode;
        }
        if (cityInput !== null) {
            cityInput.value = address.city;
        }
        if (countryInput !== null) {
            countryInput.value = address.country;
        }
    },

    clearMarkers: function() {
        for (var i = 0; i < this.markers.length; i++) {
            this.markers[i].remove();
        }
        this.markers = [];
    },
    
    showError: function(error) {
        this.closeMap();
        alert(error);
    },

    on: function(elSelector, eventName, selector, fn) {
        var element = document.querySelector(elSelector);

        element.addEventListener(eventName, function(event) {
            var possibleTargets = element.querySelectorAll(selector);
            var target = event.target;

            for (var i = 0, l = possibleTargets.length; i < l; i++) {
                var el = target;
                var p = possibleTargets[i];

                while(el && el !== element) {
                    if (el === p) {
                        return fn.call(p, event);
                    }

                    el = el.parentNode;
                }
            }
        });
    }
};
